/********************************************************************
Function definitions from Boundaries.h
********************************************************************/

#include "Boundaries.h"

void Boundaries::createRombus(int width, int x, int y, std::vector<VECTOR_2D<int>>& indices, std::vector<VECTOR_2D<double>>& normals)
{
	if(width % 2 == 0)		//Expect only odd numbers.
		width++;

	int I, J;				//To keep track of the column and row grid nodes.
	const double component = 1.0/sqrt(2.0);		//One over square root of two: used very often here for the normals.
	const int half = floor(width/2);

	J = 1;
	while(J < half)			//Create lateral boundaries symmetrically and at the same time.
	{
		I = floor( (width-2*J)/2 );		//Reduce the width as we move far from center.

		//Create the top two points.
		VECTOR_2D<int> index1(x-I, y+J);
		VECTOR_2D<double> normal1(-component, component);

		VECTOR_2D<int> index2(x+I, y+J);
		VECTOR_2D<double> normal2(component, component);

		//Create the bottom two points.
		VECTOR_2D<int> index3(x-I, y-J);
		VECTOR_2D<double> normal3(-component, -component);

		VECTOR_2D<int> index4(x+I, y-J);
		VECTOR_2D<double> normal4(component, -component);

		//Add the four indices to the list.
		indices.push_back( index1 );
		indices.push_back( index2 );
		indices.push_back( index3 );
		indices.push_back( index4 );

		//Add the four normals to the list (in the same order to make them correspond) to indices.
		normals.push_back( normal1 );
		normals.push_back( normal2 );
		normals.push_back( normal3 );
		normals.push_back( normal4 );

		J++;
	}

	//Finally, take the four corner points in the NEWS directions.
	VECTOR_2D<int> indexRight(x+half, y);
	VECTOR_2D<double> normalRight(1, 0);

	VECTOR_2D<int> indexLeft(x-half, y);
	VECTOR_2D<double> normalLeft(-1, 0);

	VECTOR_2D<int> indexTop(x, y+half);
	VECTOR_2D<double> normalTop(0, 1);

	VECTOR_2D<int> indexBottom(x, y-half);
	VECTOR_2D<double> normalBottom(0, -1);

	//Add the four indices to the list.
	indices.push_back( indexRight );
	indices.push_back( indexLeft );
	indices.push_back( indexTop );
	indices.push_back( indexBottom );

	//Add the four normals to the list.
	normals.push_back( normalRight );
	normals.push_back( normalLeft );
	normals.push_back( normalTop );
	normals.push_back( normalBottom );
}