/********************************************************************
Functions for SnowBall class.
********************************************************************/

#include "SnowBall.h"

/************** Constructor *****************/
SnowBall::SnowBall(const VECTOR_2D<double>& centerInput, const VECTOR_2D<double>& v, double radiusInput, double area)
{
	assert(centerInput(0)>0 && centerInput(0)<1);			//Validate location for center.
	assert(centerInput(1)>0 && centerInput(1)<1);
	assert(radiusInput>0 && radiusInput<0.25);				//Check that the diameter is smaller than the size of the world frame.
	assert(area > (double)0 && area < radiusInput/10.0);	//Particle area must be positive.

	PI = 3.14159265358979323846;
	center = centerInput;
	velocity = v;
	radius = radiusInput;
	numberOfParticles = floor(PI*radius*radius / area);		//Get number of particles as area_of_circle/area_of_particle.

	createSnowBall();
}

/******** Function to create the snow ball once that the boundaries have been created *********/
void SnowBall::createSnowBall()
{
	int I = 0;
	while( I < numberOfParticles)					//Execute until we have the expected number of particles.
	{
		double xRand, yRand;						//Generate two random coordinates.
		xRand = -radius + RANDOM_REAL_NUMBER<double>::Random_Number()*radius*2.0;
		yRand = -radius + RANDOM_REAL_NUMBER<double>::Random_Number()*radius*2.0;
		if(xRand*xRand + yRand*yRand < radius*radius)	//Coordinates inside the circle?.
		{
			double x = xRand + center(0);	//Compute the x coordinate in world frame.
			double y = yRand + center(1);	//Y coordinate in world frame.

			Particle p(x, y, velocity(0), velocity(1));

			particles.push_back( p );				//Put all particles in a dynamic vector.

			I++;
			std::cout<< particles.size() << " new particles have been created in a snow ball." << std::endl;
		}
	}
}

/******** Function to get a reference to the constant list of particles ********/
const std::vector<Particle>& SnowBall::getParticles()
{
	return particles;
}

/****************************** Destructor **********************/
SnowBall::~SnowBall(void)
{
}
