#include <Windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include "gl/glut.h"
#include <iostream>
#include "MPM_DRIVER.h"
#include "SnowBall.h"
#include <vector>
#include "image.h"

#define PI 3.14159265

typedef double T;
typedef VECTOR_3D<T> TV;

//Generic viewer data
int winWidth,winHeight;
T costheta,sintheta;
MATRIX_3X3<T> ACCUMULATED=MATRIX_3X3<T>::Identity();
TV tsl(0.0, 0.0, 0.0);   //original eye position
const TV def_eye(0.0, 0.0, .2*5.0);   //original eye position
const TV def_ori(0.0, 1.0, 0.0);   //original orientation of camera
TV eye(def_eye);   //original eye position
TV ori(def_ori);   //original orientation of camera
TV u(0.0, 0.0, 5.0);     //current position
TV v1(0.0, 0.0, 5.0);    //last position
T one_over_sqrt_3=(T)1/sqrt((T)3);
T rotation_scale=(T)20;
T zoom_scale=(T)0.2;
T shift_scale=(T)0.05;

//MPM related viewer data
T vector_scale=(T).05;
bool draw_particle_velocities=true;
bool draw_particles=true;
bool draw_eulerian_dofs=true;
bool draw_grid=false;
int current_frame=0;
int last_frame=0;
int m_mpm_grid=0;						//Number of cells (rows).
int n_mpm_grid=0;						//Number of cells (columns).
int num_mpm_particles=0;				//Number of particles.
T dx_mpm_grid=(T)0;
VECTOR<T>* particle_x;					//Particle positions x_p.
VECTOR<T>* particle_v;					//Particle velocities v_p.
VECTOR<T>* eulerian_v;					//Velocity at the grid nodes.
VECTOR<T>* eulerian_m;					//Mass at the grid nodes.
VECTOR<int>* particle_cell_indices;		//An array of  imin jmin indices for each particle.

GLfloat light_diffuse[] = {1.0, 1.0, 1.0, 1.0};  /* Diffuse light. */
GLfloat light_ambient[] = {1.0, 1.0, 1.0, 1.0};  /* Ambient light. */
GLfloat light_position[] = {1.0, 1.0, 1.0, 0.0};  /* Infinite light location. */

bool trackingMouseRotate=false;
bool trackingMouseZoom=false;
bool trackingMouseShift=false;
bool playback_mode=false;

//Geometry to render
MPM_DRIVER<double>* mpm;

//Images
GLuint texNames[2];
GL_Image2D Img1("background.bmp");	//Getting the path to the textures.
GL_Image2D Img2("background.bmp");

void processNormalKeys(unsigned char key, int x,int y){
    if(key==27)
        exit(0);
	if(key=='r'||key=='R') current_frame=0;
	if(key=='p'||key=='P') playback_mode=!playback_mode;
	if(key=='e'||key=='E') draw_eulerian_dofs=!draw_eulerian_dofs;
	if(key=='g'||key=='G') draw_grid=!draw_grid;
	if(key=='s') vector_scale*=(T).9;
	if(key=='S') vector_scale*=(T)1.1;
    if(key=='z'||key=='-')
        eye = eye*1.05;
    if(key=='Z'||key=='+')
        eye = eye*0.95;
	if(key==','||key=='<'){
		if(!playback_mode){
			current_frame-=1;
			if(current_frame<0) current_frame=0;}}
	if(key=='.'||key=='>'){
		if(!playback_mode){
			current_frame+=1;}}
    if(key=='i'){
        tsl=TV();
        eye=def_eye;
        ori=def_ori;
        ACCUMULATED=MATRIX_3X3<T>::Identity();}
}

void processSpecialKeys(int key, int x, int y){
    if(key==GLUT_KEY_UP)   eye = eye*0.99;
    if(key==GLUT_KEY_DOWN) eye = eye*1.01;
}

void Draw_Vector(VECTOR_2D<T> location,VECTOR_2D<T> vector,VECTOR_3D<T> color=VECTOR_3D<T>((T)1,(T)0,(T)0)){
	
	MATRIX_2X2<T> givens=MATRIX_2X2<T>::Givens_Rotation(vector);
	T length=vector_scale*vector.Magnitude();
	VECTOR_2D<T> arrow_head_top=givens.Transposed()*VECTOR_2D<T>((T).75*length,(T).25*length)+location;
	VECTOR_2D<T> arrow_head_bottom=givens.Transposed()*VECTOR_2D<T>((T).75*length,-(T).25*length)+location;
	
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHTING);
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	glLineWidth(2);
	glBegin(GL_LINES);
	glColor3f(color.x(),color.y(),color.z());
	VECTOR_2D<T> end=location+vector_scale*vector;
	glVertex3f(location.x(),location.y(),0);
	glVertex3f(end.x(),end.y(),0);
	glEnd();
	glBegin(GL_LINES);
	//glColor3f(color.x(),color.y(),color.z());
	glVertex3f(end.x(),end.y(),0);
	glVertex3f(arrow_head_top.x(),arrow_head_top.y(),0);
	glEnd();
	glBegin(GL_LINES);
	//glColor3f(color.x(),color.y(),color.z());
	glVertex3f(end.x(),end.y(),0);
	glVertex3f(arrow_head_bottom.x(),arrow_head_bottom.y(),0);
	glEnd();
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
}

void Initialize_MPM_Data_Structures(){
	int load_frame=0;
	mpm->Read_Domain_And_Frame_Info(load_frame,last_frame,m_mpm_grid,n_mpm_grid,dx_mpm_grid,num_mpm_particles);
	particle_x=new VECTOR<T>(2*num_mpm_particles);
	particle_v=new VECTOR<T>(2*num_mpm_particles);
	particle_cell_indices=new VECTOR<int>(2*num_mpm_particles);
	eulerian_v=new VECTOR<T>(2*(m_mpm_grid+1)*(n_mpm_grid+1));
	eulerian_m=new VECTOR<T>((m_mpm_grid+1)*(n_mpm_grid+1));
	mpm->Read_Frame(load_frame,*particle_x,*particle_v,*particle_cell_indices,*eulerian_v,*eulerian_m);
}

void Delete_MPM_Data_Structures(){
	delete particle_x;
	delete particle_v;
	delete particle_cell_indices;
	delete eulerian_v;
	delete eulerian_m;
}

void Load_Frame_From_File(){
	if(playback_mode) current_frame++;
	int attempt_to_read=current_frame;
	mpm->Read_Domain_And_Frame_Info(current_frame,last_frame,m_mpm_grid,n_mpm_grid,dx_mpm_grid,num_mpm_particles);
	mpm->Read_Frame(current_frame,*particle_x,*particle_v,*particle_cell_indices,*eulerian_v,*eulerian_m);
	if(attempt_to_read!=current_frame && playback_mode) current_frame=0;
}

void displayText(float x, float y, float r, float g, float b, const char *string)
{
	glColor3f(r,g,b);
	glRasterPos2f(x,y);
	int len,i;
	len=(int)strlen(string);
	for(i = 0;i<len;i++) glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,string[i]);
}

///////////////////////Texture Initialization///////////////////////
void initTexture(void)
{
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
   
	glGenTextures(23, texNames);	//Generate names for textures.
	
	//Background texture object.
	glBindTexture(GL_TEXTURE_2D, texNames[0]);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, Img1.m_width, Img1.m_height, 0, GL_RGB, GL_UNSIGNED_BYTE, Img1.m_data);
    
    //Frame texture object.
	glBindTexture(GL_TEXTURE_2D, texNames[1]);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, Img2.m_width, Img2.m_height, 0, GL_RGB, GL_UNSIGNED_BYTE, Img2.m_data);
	
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	//Authomatic Generation of texture coordinates.
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	GLfloat coeffx[] = {1, 0, 0, 0};
	glTexGenfv(GL_S, GL_OBJECT_PLANE, coeffx);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	GLfloat coeffy[] = {0, 1, 0, 0};
	glTexGenfv(GL_T, GL_OBJECT_PLANE, coeffy);	
}

/************************* Function to draw the backgroound texture *****************************/
void drawBackground()
{
	glPushMatrix();	//Saves main state.

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);	//Set mode for texture.

	glEnable(GL_TEXTURE_2D);	//Enable textures

	////////////Drawing background/////////////
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_FILL);
	glBindTexture(GL_TEXTURE_2D, texNames[0]);
	glBegin(GL_POLYGON);
		glNormal3f(0.0, 0.0, 1.0); glTexCoord2f(0.0, 0.0); glVertex3f(-0.77, -0.77, -0.5);
		glNormal3f(0.0, 0.0, 1.0); glTexCoord2f(1.0, 0.0); glVertex3f(0.77, -0.77, -0.5);
		glNormal3f(0.0, 0.0, 1.0); glTexCoord2f(1.0, 1.0); glVertex3f(0.77, 0.77, -0.5);
		glNormal3f(0.0, 0.0, 1.0); glTexCoord2f(0.0, 1.0); glVertex3f(-0.77, 0.77, -0.5);
	glEnd();

	glDisable(GL_TEXTURE_2D);	//Disable textures

	glPopMatrix();	//Retrieves main state.
}

void renderSim(void){
	
	Load_Frame_From_File();
	
	glClearColor(0.09804f,0.18039f,0.23529f,1.0f); 
    glLoadIdentity();
    gluLookAt(eye(0)+tsl(0),eye(1)+tsl(1),eye(2)+tsl(2),tsl(0),tsl(1),tsl(2),ori(0),ori(1),ori(2));
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT); 
	
	char str[12];
	sprintf(str, "%d", current_frame);	
	displayText(-.6, .5, 1, 1, 1, str);
	
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glLineWidth(1);
	
	INDEX_2D lower_left_index(0,0,mpm->M()+1,mpm->N()+1);
	INDEX_2D upper_right_index(mpm->M(),mpm->N(),mpm->M()+1,mpm->N()+1);
	
	VECTOR_2D<T> center_of_grid=(T).5*(mpm->Node(lower_left_index)+mpm->Node(upper_right_index));
	
	//draw the grid
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHTING);
	glColor3f(0.8,0.8,0.85);
	if(draw_grid){
		for(int i=0;i<mpm->M()+1;i++){
			INDEX_2D node_index0(i,0,mpm->M()+1,mpm->N()+1);
			VECTOR_2D<T> x0=mpm->Node(node_index0)-center_of_grid;
		
			INDEX_2D node_indexM(i,mpm->M(),mpm->M()+1,mpm->N()+1);
			VECTOR_2D<T> xM=mpm->Node(node_indexM)-center_of_grid;
		
			glBegin(GL_LINES);
			glVertex3f(x0.x(),x0.y(),-1);
			glVertex3f(xM.x(),xM.y(),-1);
			glEnd();}	
		for(int j=0;j<mpm->N()+1;j++){
			INDEX_2D node_index0(0,j,mpm->M()+1,mpm->N()+1);
			VECTOR_2D<T> x0=mpm->Node(node_index0)-center_of_grid;
		
			INDEX_2D node_indexN(mpm->N(),j,mpm->M()+1,mpm->N()+1);
			VECTOR_2D<T> xN=mpm->Node(node_indexN)-center_of_grid;
		
			glBegin(GL_LINES);
			glVertex3f(x0.x(),x0.y(),-1);
			glVertex3f(xN.x(),xN.y(),-1);
			glEnd();}}
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	
	//draw the particles
	if(draw_particles){
		glDisable(GL_LIGHT0);
		glDisable(GL_LIGHTING);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		//glEnable(GL_BLEND);
		//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		//glEnable(GL_POINT_SMOOTH);
		//glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);

		glColor3f(1.0,1.0,1.0);
		glPointSize(5);
		glBegin(GL_POINTS);

		for(int p=0;p<num_mpm_particles;p++){
			T x=(*particle_x)(2*p);T y=(*particle_x)(2*p+1);
			glVertex3f(x-center_of_grid.x(),y-center_of_grid.y(),0);}
		glEnd();

		//glDisable(GL_POINT_SMOOTH);
		//glBlendFunc(GL_NONE, GL_NONE);
		glDisable(GL_BLEND); 
		glEnable(GL_LIGHT0);
		glEnable(GL_LIGHTING);}
	
	if(draw_eulerian_dofs)
	{
		glDisable(GL_LIGHT0);
		glDisable(GL_LIGHTING);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		glPointSize(5);
		glBegin(GL_POINTS);
		/*for(int p=0;p<num_mpm_particles;p++){
			glColor3f(0.7882,0.8824,1.0);
			int i=(*particle_cell_indices)(2*p);int j=(*particle_cell_indices)(2*p+1);
			int imin=i-1;int jmin=j-1;			
			for(int is=0;is<4;is++){
				for(int js=0;js<4;js++){
					INDEX_2D node_index(imin+is,jmin+js,mpm->Node_Grid().Number_Nodes_Per_X_Dimension(),mpm->Node_Grid().Number_Nodes_Per_Y_Dimension());
					VECTOR_2D<T> node_position=mpm->Node(node_index);
					glVertex3f(node_position.x()-center_of_grid.x(),node_position.y()-center_of_grid.y(),0);}}}*/
		
		for(int i=0;i<mpm->M()+1;i++){
			for(int j=0;j<mpm->M()+1;j++){
				if(mpm->Is_Dirichlet(i,j)){
					glColor3f(0.0f, 0.62745f, 0.60784f);
					INDEX_2D node_index(i,j,mpm->Node_Grid().Number_Nodes_Per_X_Dimension(),mpm->Node_Grid().Number_Nodes_Per_Y_Dimension());
					VECTOR_2D<T> node_position=mpm->Node(node_index);
					glVertex3f(node_position.x()-center_of_grid.x(),node_position.y()-center_of_grid.y(),0);}}}
		
		glEnd();
		glEnable(GL_LIGHT0);
		glEnable(GL_LIGHTING);
	
	//draw node velocities
	//for(int i=0;i<mpm->Node_Grid().Number_Nodes_Per_X_Dimension();i++){
	// for(int j=0;j<mpm->Node_Grid().Number_Nodes_Per_Y_Dimension();j++){
	//	 if(mpm->Is_Dirichlet(i, j))
	//	 {
	//		INDEX_2D index(i,j,mpm->Node_Grid().Number_Nodes_Per_X_Dimension(),mpm->Node_Grid().Number_Nodes_Per_Y_Dimension());
	//		VECTOR_2D<T> x=mpm->Node(index)-center_of_grid;
	//		//VECTOR_2D<T> v((*eulerian_v)(2*index.Index()),(*eulerian_v)(2*index.Index()+1));
	//		VECTOR_2D<T> normal = mpm->Get_Dirichlet_Normal(i, j);
	//		VECTOR_3D<T> color(1.0, 0.0, 0.0);
	//		Draw_Vector(x,normal, color);
	//	 }
	// }
	//}

	//draw particle velocities
	/*if(draw_particle_velocities && draw_particles){
	 for(int p=0;p<flip->Particle_Positions().Size();p++){
	 VECTOR_2D<T> x=flip->Particle_Positions()(p);VECTOR_2D<T> v=flip->Particle_Velocities()(p);
	 Draw_Vector(x-center_of_grid,v,VECTOR_3D<T>((T)1,(T)0,(T)1));}}*/	
	}

	//Draw the background texture.
	drawBackground();

	//visualize the density
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHTING);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA); 
	T max_density=(*eulerian_m).Max();
	for(int i=0;i<mpm->Node_Grid().Number_Nodes_Per_X_Dimension();i++){
		for(int j=0;j<mpm->Node_Grid().Number_Nodes_Per_Y_Dimension();j++){
			INDEX_2D index(i,j,mpm->Node_Grid().Number_Nodes_Per_X_Dimension(),mpm->Node_Grid().Number_Nodes_Per_Y_Dimension());
			VECTOR_2D<T> x_node=mpm->Node(index)-center_of_grid;
			T dx=mpm->Node_Grid().Dx();
			T density_scale=(*eulerian_m)(index)/max_density;
			glBegin(GL_POLYGON);
			glColor4f(0.90985f, 0.91961f, 0.91567f, density_scale*2.0);
			//glColor3f(0, density_scale, 0);
			glVertex3f(x_node.x()-(T).5*dx,x_node.y()-(T).5*dx,0);
			//glColor3f(0, density_scale, 0);
			glVertex3f(x_node.x()+(T).5*dx,x_node.y()-(T).5*dx,0);
			//glColor3f(0, density_scale, 0);
			glVertex3f(x_node.x()+(T).5*dx,x_node.y()+(T).5*dx,0);
			//glColor3f(0, density_scale, 0);
			glVertex3f(x_node.x()-(T).5*dx,x_node.y()+(T).5*dx,0);
			glEnd();
		}
	}
	glDisable(GL_BLEND);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	
	glutSwapBuffers();
	
}

void track_eye(TV local_u, TV local_v1){
    TV local_v2,local_v3;
    local_v1=ACCUMULATED*local_v1;
    local_v1.Normalize();
    local_u=ACCUMULATED*local_u;
    local_u.Normalize();
    local_v3=TV::Cross_Product(local_v1,local_u);
    local_v3.Normalize();
    local_v2=local_u-local_v1.Dot_Product(local_v1,local_u)*local_v1;
    local_v2.Normalize();
    
    costheta = local_v1.Dot_Product(local_v1, local_u);   //use dot product, local_v1 and local_u have already been normalized
    sintheta = sqrt(1-costheta*costheta);   
    
    MATRIX_3X3<T> Q(local_v1,local_v2,local_v3);
    MATRIX_3X3<T> R(costheta,-sintheta,0,sintheta,costheta,0,0,0,1);
    eye=Q*R*Q.Transposed()*eye;
    ori=Q*R*Q.Transposed()*ori;
    ACCUMULATED=Q*R*Q.Transposed()*ACCUMULATED;
}

void startMotionZoom(int x,int y){
    trackingMouseZoom=true;
    v1(0)=rotation_scale*(((T)2*(T)x-(T)winWidth)/(T)winWidth)*one_over_sqrt_3;
    v1(1)=rotation_scale*(((T)winHeight-(T)2*(T)y)/(T)winHeight)*one_over_sqrt_3;
}

void stopMotionZoom(int x,int y){
    trackingMouseZoom=false;
    v1(0)=u(0);
    v1(1)=u(1);
}

void startMotionShift(int x,int y){
    trackingMouseShift=true;
    v1(0)=rotation_scale*(((T)2*(T)x-(T)winWidth)/(T)winWidth)*one_over_sqrt_3;
    v1(1)=rotation_scale*(((T)winHeight-(T)2*(T)y)/(T)winHeight)*one_over_sqrt_3;
}

void stopMotionShift(int x,int y){
    trackingMouseShift=false;
    v1(0)=u(0);
    v1(1)=u(1);
}

void mouseMotion(int x,int y){
    u(0)=rotation_scale*(((T)2*(T)x-(T)winWidth)/(T)winWidth)*one_over_sqrt_3;
    u(1)=rotation_scale*(((T)winHeight-(T)2*(T)y)/(T)winHeight)*one_over_sqrt_3;
	
    if(trackingMouseRotate){
        if(fabs(v1(0)-u(0))+fabs(v1(1)-u(1))>0.05) {
            track_eye(u,v1);
            v1(0)=u(0);
            v1(1)=u(1);}} 
    if(trackingMouseZoom){
        if(fabs(v1(1)-u(1))>0.05) {
            eye=eye*exp((u(1)-v1(1))*zoom_scale);
            v1(0)=u(0);
            v1(1)=u(1);}} 
    if(trackingMouseShift){
        if(fabs(v1(0)-u(0))+fabs(v1(1)-u(1))>0.05) {
            tsl=tsl+ACCUMULATED*(v1-u)*eye.Magnitude()*shift_scale;
            v1(0)=u(0);
            v1(1)=u(1);}} 
}

void mouseButton(int button, int state, int x, int y){

    if(button==GLUT_RIGHT_BUTTON){
		switch(state){
			case GLUT_DOWN:
				startMotionZoom(x,y);
				break;
			case GLUT_UP:
				stopMotionZoom(x,y);
            break;}}
    if(button==GLUT_MIDDLE_BUTTON){
		switch(state){
			case GLUT_DOWN:
				startMotionShift(x,y);
				break;
			case GLUT_UP:
				stopMotionShift(x,y);
            break;}}
}

void Advance_One_Time_Step(){
    mpm->Advance_One_Time_Step();
}

void display(void){
    Advance_One_Time_Step();
    renderSim();
}

void reshape(int width, int height){
    winWidth=glutGet(GLUT_WINDOW_WIDTH);
    winHeight=glutGet(GLUT_WINDOW_HEIGHT);
    glViewport(0,0,(GLsizei)width,(GLsizei)height);
    glMatrixMode(GL_PROJECTION); 
    glLoadIdentity(); 
    gluPerspective(60,(GLfloat)width/(GLfloat)height,1.0,100.0);
    glMatrixMode(GL_MODELVIEW);
}

int main (int argc, char **argv) {
	
    //set up grid resolution and initial particles
	int m=64,n=64;										//Number of cells in the grid.
	T xmax=(T)1,xmin=(T)0;
	T ymin=(T)0;
	T dx=(xmax-xmin)/(T)m;
	T dt=(T)1/(T)30;									//Time step.
	GRID_2D<T> node_grid(m,n,dx,xmin,ymin);				//Creates the grid of nodes with m x n cells.
	T particle_density=(T)1;
	T dx_particle = (T)0.0072; 
	T area = dx_particle * dx_particle;

	//Create a snowball 1.
	T radius1 = 0.1;
	VECTOR_2D<T> center1(0.5, 0.47);
	VECTOR_2D<T> velocity1(0, 0);
	SnowBall snowBall1(center1, velocity1, radius1, area);

	std::vector<Particle> particles1 = snowBall1.getParticles();		//Get all particles of this snowball in one vector.
	int numberOfParticles1 = particles1.size();
	
	//Create a snowball 2.
	T radius2 = 0.085;
	VECTOR_2D<T> center2(0.49, 0.655);
	VECTOR_2D<T> velocity2(0, 0);
	SnowBall snowBall2(center2, velocity2, radius2, area);

	std::vector<Particle> particles2 = snowBall2.getParticles();		//Get particles of snowball 2.
	int numberOfParticles2 = particles2.size();
	
	//Create a snowball 3.
	T radius3 = 0.1;
	VECTOR_2D<T> center3(0.75, 0.85);
	VECTOR_2D<T> velocity3(-0.5, 0.025);
	SnowBall snowBall3(center3, velocity3, radius3, area);

	std::vector<Particle> particles3 = snowBall3.getParticles();		//Get particles of snowball 3.
	int numberOfParticles3 = particles3.size();
	
	int totalParticles = numberOfParticles1 + numberOfParticles2 + numberOfParticles3; 
	VECTOR<T> particle_area(totalParticles);							//Allocate space for arrays.
	VECTOR<T> particle_mass(totalParticles);
	VECTOR<T> particle_positions(totalParticles*2);
	VECTOR<T> particle_velocities(totalParticles*2);

	//Copy positions, velocities, areas, and mass of particles1 into cummulative vectors.
	for( int I=0; I<numberOfParticles1; I++ )
	{
		particle_area(I) = area;										//Store the area.
		particle_mass(I) = particle_density * area;						//Store the mass.

		particle_positions(2*I) = particles1[I].getPosition().x();		//Store the positions.
		particle_positions(2*I+1) = particles1[I].getPosition().y();

		particle_velocities(2*I) = particles1[I].getVelocity().x();		//Store the velocities.
		particle_velocities(2*I+1) = particles1[I].getVelocity().y();
	}
	
	//Copy positions, velocities, areas, and mass of particles2 into cummulative vectors.
	int bigIndex;
	for( int I=0; I<numberOfParticles2; I++ )
	{
		bigIndex = I + numberOfParticles1;										//Cummulative index.
		particle_area(bigIndex) = area;											//Store the area.
		particle_mass(bigIndex) = particle_density * area;						//Store the mass.

		particle_positions(2*bigIndex) = particles2[I].getPosition().x();		//Store the positions.
		particle_positions(2*bigIndex+1) = particles2[I].getPosition().y();

		particle_velocities(2*bigIndex) = particles2[I].getVelocity().x();		//Store the velocities.
		particle_velocities(2*bigIndex+1) = particles2[I].getVelocity().y();
	}
	
	//Copy positions, velocities, areas, and mass of particles3 into cummulative vectors.
	for( int I=0; I<numberOfParticles3; I++ )
	{
		bigIndex = I + numberOfParticles1 + numberOfParticles2;					//Cummulative index.
		particle_area(bigIndex) = area;											//Store the area.
		particle_mass(bigIndex) = particle_density * area;						//Store the mass.

		particle_positions(2*bigIndex) = particles3[I].getPosition().x();		//Store the positions.
		particle_positions(2*bigIndex+1) = particles3[I].getPosition().y();

		particle_velocities(2*bigIndex) = particles3[I].getVelocity().x();		//Store the velocities.
		particle_velocities(2*bigIndex+1) = particles3[I].getVelocity().y();
	}
	
	//core MPM code
	MPM_DRIVER<T> mpm_driver(node_grid,particle_positions,particle_velocities,particle_mass,particle_area,dt,dx_particle);
	mpm=&mpm_driver;
	//setup visualize of MPM
	Initialize_MPM_Data_Structures();
	
	//core opengl loop
	glutInit(&argc,argv); 
    glutInitDisplayMode(GLUT_DOUBLE|GLUT_DEPTH); 
    glutInitWindowSize(1024,768); 
    glutInitWindowPosition(100,100);
    glutCreateWindow("MPM Visualization"); 
    
    winWidth=glutGet(GLUT_WINDOW_WIDTH);
    winHeight=glutGet(GLUT_WINDOW_HEIGHT);

	//Initialize textures.
	initTexture();
    
    glutDisplayFunc(display); 
    glutIdleFunc(display); 
    glutMouseFunc(mouseButton);
    glutMotionFunc(mouseMotion);
    glutKeyboardFunc(processNormalKeys);
    glutSpecialFunc(processSpecialKeys);
    glutReshapeFunc(reshape);
    
    glEnable(GL_DEPTH_TEST); 
	
    glLightfv(GL_LIGHT0,GL_DIFFUSE,light_diffuse);
    glLightfv(GL_LIGHT0,GL_AMBIENT,light_ambient);
    glLightfv(GL_LIGHT0,GL_POSITION,light_position);
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHTING);
    glPolygonOffset(1,1);
    glEnable(GL_POLYGON_OFFSET_FILL);
	
    glutMainLoop();
	
	//clean up
	Delete_MPM_Data_Structures();
}

